module separate_mod

  implicit none

  include "fftw3.f"

  integer, private                                :: n1, n2, n1e, n2e, npad
  integer*8, private                              :: planf, planb
  real, private, dimension(:,:), allocatable      :: wt
  complex, private, dimension(:,:), allocatable   :: in, out

  contains

  subroutine separate_init(n1_in, n2_in, d1, d2, dist1, dist2)

    integer               :: n1_in, n2_in
    real                  :: d1, d2, dk1, dk2
    real                  :: dist, dist1, dist2, pi
    integer               :: i, j

    n1 = n1_in
    n2 = n2_in

    npad = n1+n2

    n1e = n1 + 2*npad
    n2e = n2 + 2*npad
    dk1 = 1./(n1e*d1)
    dk2 = 1./(n2e*d2)

    pi = 3.14159265358979323846

    allocate( wt(n1e, n2e), in(n1e, n2e), out(n1e, n2e) )

    wt = 0.
    in = 0.
    out = 0.

    call sfftw_plan_dft_2d(planf, n1e, n2e, in, out, FFTW_FORWARD, FFTW_ESTIMATE)
    call sfftw_plan_dft_2d(planb, n1e, n2e, out, in, FFTW_BACKWARD, FFTW_ESTIMATE)

    do j=1,n2e/2
      do i=1,n1e/2
        dist = sqrt((1.-i)*dk1*(1.-i)*dk1+(1.-j)*dk2*(1.-j)*dk2)
        if(dist > dist2) then
          wt(i,j) = 0.
        elseif(dist > dist1) then
          wt(i,j) = cos(pi/2.*(dist-dist1)/(dist2-dist1))*cos(pi/2.*(dist-dist1)/(dist2-dist1))
        else
          wt(i,j) = 1.
        endif
      enddo
    enddo

    do j=1,n2e/2
      do i=n1e/2,n1e
        dist = sqrt((n1e-i)*dk1*(n1e-i)*dk1+(1.-j)*dk2*(1.-j)*dk2)
        if(dist > dist2) then
          wt(i,j) = 0.
        elseif(dist > dist1) then
          wt(i,j) = cos(pi/2.*(dist-dist1)/(dist2-dist1))*cos(pi/2.*(dist-dist1)/(dist2-dist1))
        else
          wt(i,j) = 1.
        endif
      enddo
    enddo

    do j=n2e/2,n2e
      do i=1,n1e/2
        dist = sqrt((1.-i)*dk1*(1.-i)*dk1+(n2e-j)*dk2*(n2e-j)*dk2)
        if(dist > dist2) then
          wt(i,j) = 0.
        elseif(dist > dist1) then
          wt(i,j) = cos(pi/2.*(dist-dist1)/(dist2-dist1))*cos(pi/2.*(dist-dist1)/(dist2-dist1))
        else
          wt(i,j) = 1.
        endif
      enddo
    enddo

    do j=n2e/2,n2e
      do i=n1e/2,n1e
        dist = sqrt(1.*(n1e-i)*dk1*(n1e-i)*dk1+(n2e-j)*dk2*(n2e-j)*dk2)
        if(dist > dist2) then
          wt(i,j) = 0.
        elseif(dist > dist1) then
          wt(i,j) = cos(pi/2.*(dist-dist1)/(dist2-dist1))*cos(pi/2.*(dist-dist1)/(dist2-dist1))
        else
          wt(i,j) = 1.
        endif
      enddo
    enddo

  end subroutine

  subroutine separate_smooth(data, data2)
    real, dimension(n1, n2)  :: data, data2

    in = 0.
    in(1+npad:n1+npad, 1+npad:n2+npad) = data
    call extend(in, npad)

    out = 0.
    call sfftw_execute_dft(planf, in, out)

    out = out * wt

    in = 0.
    call sfftw_execute_dft(planb, out, in)

    data2 = in(1+npad:n1+npad, 1+npad:n2+npad)
    data2 = data2 / n1e / n2e

  end subroutine

  subroutine separate_rough(data, data2)
    real, dimension(n1, n2)  :: data, data2

    in = 0.
    in(1+npad:n1+npad, 1+npad:n2+npad) = data
    call extend(in, npad)

    out = 0.
    call sfftw_execute_dft(planf, in, out)

    out = out * (1.-wt)

    in = 0.
    call sfftw_execute_dft(planb, out, in)

    data2 = in(1+npad:n1+npad, 1+npad:n2+npad)
    data2 = data2 / n1e / n2e

  end subroutine

  subroutine extend(model, crop)
    integer                       :: crop
    complex, dimension(n1e, n2e)  :: model
    integer                       :: iz, ix

    do iz=1,crop
      model(iz,:) = model(1+crop,:)
    enddo

    do iz=n1e-crop+1,n1e
      model(iz,:) = model(n1e-crop,:)
    enddo

    do ix=1,crop
      model(:,ix) = model(:,1+crop)
    enddo

    do ix=n2e-crop+1,n2e
      model(:,ix) = model(:,n2e-crop)
    enddo

  end subroutine 

end module 
